# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bridgebots_tools']

package_data = \
{'': ['*']}

install_requires = \
['bridgebots==0.0.11a1', 'click>=8.0.3,<9.0.0']

setup_kwargs = {
    'name': 'bridgebots-tools',
    'version': '0.0.4',
    'description': 'Tools which utilize the Bridgebots package to process data',
    'long_description': None,
    'author': 'Forrest Rice',
    'author_email': 'forrest.d.rice@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
